@@ -5,11 +5,12 @@ use std::time::{SystemTime, UNIX_EPOCH};
 
 use faster_hex::hex_string;
 
-use serde_json::json;
+use linux_audit_parser::*;
 
-use crate::constants::{msg_type::*, ARCH_NAMES, SYSCALL_NAMES};
+use serde::Serialize;
+
+use crate::constants::{ARCH_NAMES, SYSCALL_NAMES};
 use crate::label_matcher::LabelMatcher;
-use crate::parser::{parse, ParseError};
 use crate::proc::{self, ContainerInfo, ProcTable, Process, ProcessKey};
 #[cfg(all(feature = "procfs", target_os = "linux"))]
 use crate::procfs;
@@ -42,6 +43,10 @@ pub struct Settings {
 
     pub label_exe: Option<LabelMatcher>,
     pub unlabel_exe: Option<LabelMatcher>,
+    pub label_argv: Option<LabelMatcher>,
+    pub unlabel_argv: Option<LabelMatcher>,
+    pub label_argv_bytes: usize,
+    pub label_argv_count: usize,
     pub label_script: Option<LabelMatcher>,
     pub unlabel_script: Option<LabelMatcher>,
 
@@ -70,6 +75,10 @@ impl Default for Settings {
             drop_translated: false,
             label_exe: None,
             unlabel_exe: None,
+            label_argv: None,
+            unlabel_argv: None,
+            label_argv_bytes: 4096,
+            label_argv_count: 32,
             label_script: None,
             unlabel_script: None,
             filter_keys: HashSet::new(),
@@ -116,7 +125,7 @@ const EXPIRE_DONE_TIMEOUT: u64 = 120_000;
 /// generate translation of SocketAddr enum to a format similar to
 /// what auditd log_format=ENRICHED produces
 #[cfg(target_os = "linux")]
-fn add_translated_socketaddr(rv: &mut Record, sa: SocketAddr) {
+fn add_translated_socketaddr(rv: &mut Body, sa: SocketAddr) {
     let mut m: Vec<(Key, Value)> = Vec::with_capacity(5);
     match sa {
         SocketAddr::Local(sa) => {
@@ -197,7 +206,7 @@ fn add_translated_socketaddr(rv: &mut Record, sa: SocketAddr) {
 /// As an extra sanity check, exe is compared with normalized
 /// PATH.name. If they are equal, no script is returned.
 #[cfg(all(feature = "procfs", target_os = "linux"))]
-fn path_script_name(path: &Record, pid: u32, ppid: u32, cwd: &[u8], exe: &[u8]) -> Option<NVec> {
+fn path_script_name(path: &Body, pid: u32, ppid: u32, cwd: &[u8], exe: &[u8]) -> Option<NVec> {
     use std::{
         ffi::OsStr,
         os::unix::{ffi::OsStrExt, fs::MetadataExt},
@@ -244,7 +253,7 @@ fn path_script_name(path: &Record, pid: u32, ppid: u32, cwd: &[u8], exe: &[u8])
         } else if k == "dev" {
             if let Value::Str(r, _) = v {
                 p_dev = String::from_utf8_lossy(r)
-                    .split(|c| c == ':')
+                    .split(':')
                     .filter_map(|part| u64::from_str_radix(part, 16).ok())
                     .collect::<Vec<_>>()
                     .try_into()
@@ -262,7 +271,7 @@ fn path_script_name(path: &Record, pid: u32, ppid: u32, cwd: &[u8], exe: &[u8])
 }
 
 /// Create an enriched pid entry in rv.
-fn add_record_procinfo(rec: &mut Record, key: &[u8], proc: &Process, include_names: bool) {
+fn add_record_procinfo(rec: &mut Body, key: &[u8], proc: &Process, include_names: bool) {
     let mut m: Vec<(Key, Value)> = Vec::with_capacity(4);
     match &proc.key {
         ProcessKey::Event(id) => {
@@ -347,10 +356,11 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
     /// Translates UID, GID and variants, e.g.:
     /// - auid=1000 -> AUID="user"
     /// - ogid=1000 -> OGID="user"
+    ///
     /// IDs that can't be resolved are translated into "unknown(n)".
     /// `(uint32)-1` is translated into "unset".
     #[inline(always)]
-    fn add_record_userdb(&mut self, rec: &mut Record, key: &Key, value: &Value) -> bool {
+    fn add_record_userdb(&mut self, rec: &mut Body, key: &Key, value: &Value) -> bool {
         if !self.settings.translate_userdb {
             return false;
         }
@@ -381,7 +391,7 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
         }
     }
 
-    fn add_record_uid_groups(&mut self, rec: &mut Record, key: &Key, value: &Value) {
+    fn add_record_uid_groups(&mut self, rec: &mut Body, key: &Key, value: &Value) {
         if !self.settings.enrich_uid_groups {
             return;
         }
@@ -399,7 +409,7 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
 
     /// Enrich "pid" entries using `ppid`, `exe`, `ID` (generating
     /// event id) from the shadow process table
-    fn enrich_pid(&mut self, rv: &mut Record, k: &Key, v: &Value) {
+    fn enrich_pid(&mut self, rv: &mut Body, k: &Key, v: &Value) {
         if !self.settings.enrich_pid {
             return;
         }
@@ -417,8 +427,8 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
     }
 
     /// Apply uid, gid, pid enrichment to generic records
-    fn enrich_generic(&mut self, rv: &mut Record) {
-        let mut nrv = Record::default();
+    fn enrich_generic(&mut self, rv: &mut Body) {
+        let mut nrv = Body::default();
         rv.retain(|(k, v)| {
             if self.add_record_userdb(&mut nrv, k, v) {
                 if self.settings.drop_translated {
@@ -429,13 +439,13 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
             }
             true
         });
-        rv.concat(nrv);
+        rv.extend(nrv);
     }
 
     /// Transform PROCTITLE record
     ///
     /// The flat proctitle field is turned into a list.
-    fn transform_proctitle(&mut self, rv: &mut Record) {
+    fn transform_proctitle(&mut self, rv: &mut Body) {
         let mut argv = vec![];
         rv.retain(|(k, v)| {
             match (k, v) {
@@ -463,8 +473,8 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
     }
 
     /// Enrich SOCKADDR record
-    fn enrich_sockaddr(&mut self, rv: &mut Record) {
-        let mut nrv = Record::default();
+    fn enrich_sockaddr(&mut self, rv: &mut Body) {
+        let mut nrv = Body::default();
         rv.retain(|(k, v)| match (k, v) {
             (k, Value::Str(vr, _q)) if self.settings.translate_universal => {
                 if k == "saddr" {
@@ -480,7 +490,7 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
             }
             _ => true,
         });
-        rv.concat(nrv);
+        rv.extend(nrv);
     }
 
     /// Enrich SYSCALL record
@@ -488,12 +498,12 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
     /// Add ARCH, SYSCALL, PID, PPID, SCRIPT, LABELS if appropriate
     fn enrich_syscall(
         &mut self,
-        rv: &mut Record,
+        rv: &mut Body,
         arch_sys: (Option<&'static str>, Option<&'static str>),
         current_process: &Option<Process>,
         parent: &Option<Process>,
         script: &Option<NVec>,
-        container_info: &mut Option<Record>,
+        container_info: &mut Option<Body>,
     ) {
         if let (true, (Some(arch), Some(syscall))) = (self.settings.translate_universal, arch_sys) {
             rv.push((Key::Literal("ARCH"), Value::Literal(arch)));
@@ -528,7 +538,7 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
 
             #[cfg(all(feature = "procfs", target_os = "linux"))]
             if let (true, Some(c)) = (self.settings.enrich_container, &proc.container_info) {
-                let mut ci = Record::default();
+                let mut ci = Body::default();
                 ci.push((
                     Key::Literal("ID"),
                     Value::Str(hex_string(&c.id).as_bytes(), Quote::None),
@@ -538,7 +548,7 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
         }
     }
 
-    fn transform_execve(&mut self, rv: &mut Record, current_process: &Option<Process>) {
+    fn transform_execve(&mut self, rv: &mut Body, mut current_process: &mut Option<Process>) {
         let mut argv: Vec<Value> = Vec::with_capacity(rv.len() - 1);
         rv.retain(|(k, v)| {
             match k {
@@ -576,6 +586,47 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
             }
         });
 
+        if current_process.is_some()
+            && self.settings.label_argv_count > 0
+            && self.settings.label_argv_bytes > 0
+            && (self.settings.label_argv.is_some() || self.settings.unlabel_argv.is_some())
+        {
+            let mut buf: Vec<u8> = Vec::with_capacity(self.settings.label_argv_bytes);
+
+            for arg in argv.iter().take(self.settings.label_argv_count) {
+                if !buf.is_empty() {
+                    buf.push(b' ');
+                }
+                // FIXME TryFrom<&Value> needs to be implemented in linux-audit-parser
+                let b: Vec<u8> = match arg.clone().try_into() {
+                    Ok(b) => b,
+                    Err(_) => continue,
+                };
+                if buf.len() + b.len() >= self.settings.label_argv_bytes {
+                    break;
+                }
+                buf.extend(b);
+            }
+
+            match (&mut current_process, &self.settings.label_argv) {
+                (Some(ref mut proc), Some(ref m)) => {
+                    for label in m.matches(&buf) {
+                        proc.labels.insert(label.into());
+                    }
+                }
+                _ => {}
+            }
+
+            match (&mut current_process, &self.settings.unlabel_argv) {
+                (Some(ref mut proc), Some(ref m)) => {
+                    for label in m.matches(&buf) {
+                        proc.labels.remove(label.into());
+                    }
+                }
+                _ => {}
+            }
+        }
+
         // Strip data from the middle of excessively long ARGV
         if let Some(argv_max) = self.settings.execve_argv_limit_bytes {
             let argv_size: usize = argv.iter().map(|v| 1 + v.str_len()).sum();
@@ -621,7 +672,7 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
 
         // ENV
         #[cfg(all(feature = "procfs", target_os = "linux"))]
-        if let (Some(proc), false) = (&current_process, self.settings.execve_env.is_empty()) {
+        if let (Some(ref proc), false) = (&current_process, self.settings.execve_env.is_empty()) {
             if let Ok(vars) =
                 procfs::get_environ(proc.pid, |k| self.settings.execve_env.contains(k))
             {
@@ -662,7 +713,7 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
         let mut current_process: Option<Process> = None;
         let mut parent: Option<Process> = None;
 
-        if let Some(EventValues::Single(rv)) = ev.body.get_mut(&SYSCALL) {
+        if let Some(EventValues::Single(rv)) = ev.body.get_mut(&MessageType::SYSCALL) {
             let mut proc = Process::default();
             let mut extra = 2; // pid, ppid
             if self.settings.translate_universal {
@@ -671,8 +722,9 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
             if self.settings.translate_userdb {
                 extra += 9; // uid, gid
             }
+            // FIXME
             rv.reserve(extra);
-            let mut nrv = Record::default();
+            let mut nrv = Body::default();
             let mut argv = Vec::with_capacity(4);
             rv.retain(|(k, v)| {
                 match (k, v) {
@@ -723,7 +775,7 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
                 true
             });
             rv.push((Key::Literal("ARGV"), Value::List(argv)));
-            rv.concat(nrv);
+            rv.extend(nrv);
 
             if let (Some(arch), Some(syscall)) = (arch, syscall) {
                 if let Some(an) = ARCH_NAMES.get(&arch) {
@@ -750,12 +802,11 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
                         // Process entry.
                         syscall_is_exec = sn.contains("execve");
                         parent = self.processes.get_or_retrieve(proc.ppid).cloned();
-                        let pr;
-                        if syscall_is_exec {
-                            pr = None;
+                        let pr = if syscall_is_exec {
+                            None
                         } else {
-                            pr = self.processes.get_or_retrieve(proc.pid);
-                        }
+                            self.processes.get_pid(proc.pid)
+                        };
                         match pr {
                             Some(pr) if proc.ppid == pr.ppid && proc.exe == pr.exe => {
                                 // existing, plausible process in table
@@ -805,7 +856,7 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
                         }
                         if let Some(unlabel_exe) = &self.settings.unlabel_exe {
                             for label in unlabel_exe.matches(&proc.exe.clone().unwrap()) {
-                                proc.labels.insert(label.into());
+                                proc.labels.remove(label.into());
                             }
                         }
                     }
@@ -826,8 +877,8 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
             current_process = Some(proc);
         }
 
-        if let Some(EventValues::Single(rv)) = ev.body.get_mut(&EXECVE) {
-            self.transform_execve(rv, &current_process);
+        if let Some(EventValues::Single(rv)) = ev.body.get_mut(&MessageType::EXECVE) {
+            self.transform_execve(rv, &mut current_process);
         }
 
         // Handle script enrichment
@@ -835,10 +886,14 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
         let script: Option<NVec> = match (self.settings.enrich_script, &self.settings.label_script)
         {
             (false, None) => None,
-            _ => match (&current_process, ev.body.get(&PATH), syscall_is_exec) {
+            _ => match (
+                &current_process,
+                ev.body.get(&MessageType::PATH),
+                syscall_is_exec,
+            ) {
                 (Some(proc), Some(EventValues::Multi(paths)), true) => {
                     let mut cwd = &b"/"[..];
-                    if let Some(EventValues::Single(r)) = ev.body.get(&CWD) {
+                    if let Some(EventValues::Single(r)) = ev.body.get(&MessageType::CWD) {
                         if let Some(Value::Str(rv, _)) = r.get("cwd") {
                             cwd = rv;
                         }
@@ -891,11 +946,11 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
         // Since the event may have been dropped here, don't
         // manipulate the current process below.
         let current_process = current_process;
-        let mut container_info: Option<Record> = None;
+        let mut container_info: Option<Body> = None;
 
         for tv in ev.body.iter_mut() {
             match tv {
-                (&SYSCALL, EventValues::Single(rv)) => self.enrich_syscall(
+                (&MessageType::SYSCALL, EventValues::Single(rv)) => self.enrich_syscall(
                     rv,
                     (arch_name, syscall_name),
                     &current_process,
@@ -903,11 +958,11 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
                     &script,
                     &mut container_info,
                 ),
-                (&EXECVE, EventValues::Single(_)) => {}
-                (&SOCKADDR, EventValues::Multi(rvs)) => {
+                (&MessageType::EXECVE, EventValues::Single(_)) => {}
+                (&MessageType::SOCKADDR, EventValues::Multi(rvs)) => {
                     rvs.iter_mut().for_each(|rv| self.enrich_sockaddr(rv))
                 }
-                (&PROCTITLE, EventValues::Single(rv)) => self.transform_proctitle(rv),
+                (&MessageType::PROCTITLE, EventValues::Single(rv)) => self.transform_proctitle(rv),
                 (_, EventValues::Single(rv)) => self.enrich_generic(rv),
                 (_, EventValues::Multi(rvs)) => {
                     rvs.iter_mut().for_each(|rv| self.enrich_generic(rv))
@@ -915,7 +970,7 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
             }
         }
 
-        container_info.map(|ci| ev.body.insert(CONTAINER_INFO, EventValues::Single(ci)));
+        ev.container_info = container_info;
     }
 
     /// Do bookkeeping on event, transform, emit it via the provided
@@ -939,60 +994,61 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
         let filter_raw = self.settings.filter_raw_lines.is_match(line);
 
         let skip_enriched = self.settings.translate_universal && self.settings.translate_userdb;
-        let (node, typ, id, rv) = parse(line, skip_enriched).map_err(CoalesceError::Parse)?;
-        let nid = (node.clone(), id);
+        let msg = parse(line, skip_enriched).map_err(CoalesceError::Parse)?;
+        let nid = (msg.node.clone(), msg.id);
 
         // clean out state every EXPIRE_PERIOD
         match self.next_expire {
-            Some(t) if t < id.timestamp => {
-                self.expire_inflight(id.timestamp);
-                self.expire_done(id.timestamp);
+            Some(t) if t < msg.id.timestamp => {
+                self.expire_inflight(msg.id.timestamp);
+                self.expire_done(msg.id.timestamp);
                 self.processes.expire();
-                self.next_expire = Some(id.timestamp + EXPIRE_PERIOD)
+                self.next_expire = Some(msg.id.timestamp + EXPIRE_PERIOD)
             }
-            None => self.next_expire = Some(id.timestamp + EXPIRE_PERIOD),
+            None => self.next_expire = Some(msg.id.timestamp + EXPIRE_PERIOD),
             _ => (),
         };
 
-        if typ == EOE {
+        if msg.ty == MessageType::EOE {
             if self.done.contains(&nid) {
-                return Err(CoalesceError::DuplicateEvent(id));
+                return Err(CoalesceError::DuplicateEvent(msg.id));
             }
             let ev = self
                 .inflight
                 .remove(&nid)
-                .ok_or(CoalesceError::SpuriousEOE(id))?;
+                .ok_or(CoalesceError::SpuriousEOE(msg.id))?;
             self.emit_event(ev);
-        } else if typ.is_multipart() {
+        } else if msg.ty.is_multipart() {
             // kernel-level messages
             if !self.inflight.contains_key(&nid) {
-                self.inflight.insert(nid.clone(), Event::new(node, id));
+                self.inflight
+                    .insert(nid.clone(), Event::new(msg.node, msg.id));
             }
             let ev = self.inflight.get_mut(&nid).unwrap();
             ev.filter |= filter_raw;
-            match ev.body.get_mut(&typ) {
-                Some(EventValues::Single(v)) => v.concat(rv),
-                Some(EventValues::Multi(v)) => v.push(rv),
-                None => match typ {
-                    SYSCALL => {
-                        ev.body.insert(typ, EventValues::Single(rv));
+            match ev.body.get_mut(&msg.ty) {
+                Some(EventValues::Single(v)) => v.extend(msg.body),
+                Some(EventValues::Multi(v)) => v.push(msg.body),
+                None => match msg.ty {
+                    MessageType::SYSCALL => {
+                        ev.body.insert(msg.ty, EventValues::Single(msg.body));
                     }
-                    EXECVE | PROCTITLE | CWD => {
-                        ev.body.insert(typ, EventValues::Single(rv));
+                    MessageType::EXECVE | MessageType::PROCTITLE | MessageType::CWD => {
+                        ev.body.insert(msg.ty, EventValues::Single(msg.body));
                     }
                     _ => {
-                        ev.body.insert(typ, EventValues::Multi(vec![rv]));
+                        ev.body.insert(msg.ty, EventValues::Multi(vec![msg.body]));
                     }
                 },
             };
         } else {
             // user-space messages
             if self.done.contains(&nid) {
-                return Err(CoalesceError::DuplicateEvent(id));
+                return Err(CoalesceError::DuplicateEvent(msg.id));
             }
-            let mut ev = Event::new(node, id);
+            let mut ev = Event::new(msg.node, msg.id);
             ev.filter |= filter_raw;
-            ev.body.insert(typ, EventValues::Single(rv));
+            ev.body.insert(msg.ty, EventValues::Single(msg.body));
             self.emit_event(ev);
         }
         Ok(())
@@ -1004,34 +1060,57 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
     }
 
     pub fn dump_state(&self, mut w: &mut dyn Write) -> Result<(), Box<dyn Error>> {
-        serde_json::to_writer(
+        #[derive(Serialize)]
+        struct Message<'a> {
+            #[serde(rename = "type")]
+            typ: &'static str,
+            inflight: BTreeMap<String, &'a Event<'a>>,
+            done: Vec<String>,
+            processes: &'a ProcTable,
+            next_expire: Option<u64>,
+        }
+
+        #[derive(Serialize)]
+        struct Out<'a> {
+            ts: u64,
+            message: &'a Message<'a>,
+        }
+
+        crate::json::to_writer(
             &mut w,
-            &json!({
-                "ts": SystemTime::now().duration_since(UNIX_EPOCH).unwrap().as_secs(),
-                "message": {
-                    "type": "dump_state",
-                    "label_exe": self.settings.label_exe,
-                    "inflight": self.inflight.iter().map(
-                        |(k,v)| {
+            &Out {
+                ts: SystemTime::now()
+                    .duration_since(UNIX_EPOCH)
+                    .unwrap()
+                    .as_secs(),
+                message: &Message {
+                    typ: "dump_state",
+                    inflight: self
+                        .inflight
+                        .iter()
+                        .map(|(k, v)| {
                             if let Some(node) = &k.0 {
                                 (format!("{}::{}", String::from_utf8_lossy(node), k.1), v)
                             } else {
                                 (format!("{}", k.1), v)
                             }
-                        }
-                    ).collect::<BTreeMap<_,_>>(),
-                    "done": self.done.iter().map(
-                        |v| if let Some(node ) = &v.0 {
-                            format!("{}::{}", String::from_utf8_lossy(node), v.1)
-                        } else {
-                            format!("{}", v.1)
-                        }
-                    ).collect::<Vec<_>>(),
-                    "processes": self.processes,
-                    "userdb": self.userdb,
-                    "next_expire": self.next_expire,
+                        })
+                        .collect::<BTreeMap<_, _>>(),
+                    done: self
+                        .done
+                        .iter()
+                        .map(|v| {
+                            if let Some(node) = &v.0 {
+                                format!("{}::{}", String::from_utf8_lossy(node), v.1)
+                            } else {
+                                format!("{}", v.1)
+                            }
+                        })
+                        .collect::<Vec<_>>(),
+                    processes: &self.processes,
+                    next_expire: self.next_expire,
                 },
-            }),
+            },
         )?;
         w.write_all(b"\n")?;
         w.flush()?;
@@ -1058,7 +1137,7 @@ mod test {
         String::from_utf8_lossy(&out).to_string()
     }
 
-    fn find_event<'a>(events: &[Event<'a>], id: &str) -> Option<Event<'a>> {
+    fn find_event<'a>(events: &'a [Event], id: &str) -> Option<Event<'a>> {
         events.iter().find(|e| &e.id == id).cloned()
     }
 
@@ -1209,13 +1288,15 @@ mod test {
         c.settings.translate_userdb = true;
         c.settings.translate_universal = true;
         process_record(&mut c, include_bytes!("testdata/record-login.txt")).unwrap();
-        if let EventValues::Multi(records) = &ec.borrow().as_ref().unwrap().body[&LOGIN] {
+        if let EventValues::Multi(records) =
+            &ec.borrow().as_ref().unwrap().body[&MessageType::LOGIN]
+        {
             // Check for: pid uid subj old-auid auid tty old-ses ses res UID OLD-AUID AUID
             let l = records[0].len();
             assert!(
                 l == 12,
                 "expected 12 fields, got {l}: {:?}",
-                records[0].into_iter().collect::<Vec<_>>()
+                records[0].clone().into_iter().collect::<Vec<_>>()
             );
         } else {
             panic!("expected EventValues::Multi");
@@ -1250,7 +1331,9 @@ mod test {
         )
         .unwrap();
 
-        if let EventValues::Single(record) = &ec.borrow().as_ref().unwrap().body[&SYSCALL] {
+        if let EventValues::Single(record) =
+            &ec.borrow().as_ref().unwrap().body[&MessageType::SYSCALL]
+        {
             let mut uids = 0;
             let mut gids = 0;
             for (k, v) in record {
@@ -1269,7 +1352,9 @@ mod test {
             );
         }
 
-        if let EventValues::Multi(records) = &ec.borrow().as_ref().unwrap().body[&LOGIN] {
+        if let EventValues::Multi(records) =
+            &ec.borrow().as_ref().unwrap().body[&MessageType::LOGIN]
+        {
             let mut uid = false;
             let mut old_auid = false;
             let mut auid = false;
@@ -1288,17 +1373,17 @@ mod test {
             assert!(
                 uid,
                 "missing UID: {:?}",
-                records[0].into_iter().collect::<Vec<_>>()
+                records[0].clone().into_iter().collect::<Vec<_>>()
             );
             assert!(
                 old_auid,
                 "missing OLD-AUID: {:?}",
-                records[0].into_iter().collect::<Vec<_>>()
+                records[0].clone().into_iter().collect::<Vec<_>>()
             );
             assert!(
                 auid,
                 "missing AUID: {:?}",
-                records[0].into_iter().collect::<Vec<_>>()
+                records[0].clone().into_iter().collect::<Vec<_>>()
             );
         } else {
             panic!("expected EventValues::Multi");
@@ -1376,6 +1461,43 @@ mod test {
         Ok(())
     }
 
+    #[test]
+    fn label_argv() -> Result<(), Box<dyn Error>> {
+        let ec: Rc<RefCell<Option<Event>>> = Rc::new(RefCell::new(None));
+
+        let mut c = Coalesce::new(mk_emit(&ec));
+        c.settings.label_argv = Some(LabelMatcher::new(&[(
+            r#"^\S*java .* -Dweblogic"#,
+            "weblogic",
+        )])?);
+
+        process_record(&mut c, include_bytes!("testdata/record-weblogic.txt"))?;
+
+        assert!(event_to_json(ec.borrow().as_ref().unwrap()).contains(r#"LABELS":["weblogic"]"#));
+
+        // Ensure this does not crash with long command lines
+        // TODO: check matcher behavior
+        let mut c = Coalesce::new(mk_emit(&ec));
+        c.settings.label_argv = Some(LabelMatcher::new(&[(
+            r#"/opt/app/redacted/to/protect/the/guilty/"#,
+            "protect-the-guilty",
+        )])?);
+        let buf = gen_long_find_execve();
+        process_record(&mut c, buf)?;
+        assert!(event_to_json(ec.borrow().as_ref().unwrap())
+            .contains(r#"LABELS":["protect-the-guilty"]"#));
+
+        let mut c = Coalesce::new(mk_emit(&ec));
+        c.settings.label_argv = Some(LabelMatcher::new(&[
+            (r#"^/bin/echo "#, "echo"), // this should match.
+            (r#"aaaaaaaaaa"#, "aaaa"),  // this shouldn't. argv[1] is too long for the buffer.
+        ])?);
+        process_record(&mut c, include_bytes!("testdata/record-execve-long.txt"))?;
+        assert!(event_to_json(ec.borrow().as_ref().unwrap()).contains(r#"LABELS":["echo"]"#));
+
+        Ok(())
+    }
+
     // Returns an emitter function that puts the event into an Option
     fn mk_emit<'c, 'ev: 'c>(
         ec: &'c Rc<RefCell<Option<Event<'ev>>>>,
@@ -1489,33 +1611,29 @@ mod test {
 
     #[test]
     fn filter_raw() {
-        let events: Rc<RefCell<Vec<Event>>> = Rc::new(RefCell::new(vec![]));
-
-        let mut c = Coalesce::new(mk_emit_vec(&events));
-        c.settings.filter_raw_lines = regex::bytes::RegexSet::new(&[
-            "^type=SOCKADDR (?:node=\\$*? )?msg=audit\\(\\S*?\\): saddr=01002F7661722F72756E2F6E7363642F736F636B657400",
-        ])
-        .expect("failed to compile regex");
-        c.settings.filter_first_per_process = true;
-
-        process_record(&mut c, include_bytes!("testdata/record-nscd.txt")).unwrap();
-
-        assert!(
-            !events
-                .borrow()
-                .iter()
-                .any(|e| &e.id == "1705071450.879:29498378"),
-            "nscd connect event should be filtered"
-        )
+        for (name, filter) in &[
+            ("sockaddr", "^type=SOCKADDR (?:node=\\$*? )?msg=audit\\(\\S*?\\): saddr=01002F7661722F72756E2F6E7363642F736F636B657400"),
+            ("syscall", "^type=SYSCALL (?:node=\\$*? )?msg=audit\\(.*?\\): arch=c000003e syscall=42 success=no"),
+        ] {
+            let events: Rc<RefCell<Vec<Event>>> = Rc::new(RefCell::new(vec![]));
+            let mut c = Coalesce::new(mk_emit_vec(&events));
+            c.settings.filter_raw_lines = regex::bytes::RegexSet::new(&[
+                filter
+            ])
+                .expect("failed to compile regex");
+            c.settings.filter_first_per_process = true;
+            process_record(&mut c, include_bytes!("testdata/record-nscd.txt")).unwrap();
+            assert!(
+                !events
+                    .borrow()
+                    .iter()
+                    .any(|e| &e.id == "1705071450.879:29498378"),
+                "nscd connect event should be filtered using {name}"
+            )
+        }
     }
 
-    #[test]
-    fn strip_long_argv() -> Result<(), Box<dyn Error>> {
-        let ec: Rc<RefCell<Option<Event>>> = Rc::new(RefCell::new(None));
-
-        let mut c = Coalesce::new(mk_emit(&ec));
-
-        c.settings.execve_argv_limit_bytes = Some(10000);
+    fn gen_long_find_execve() -> Vec<u8> {
         let mut buf = vec![];
         let msgid = "1663143990.204:2148478";
         let npath = 40000;
@@ -1555,6 +1673,17 @@ mod test {
             buf.extend(format!(r#" a{}="{param}""#, npath + i).bytes());
         }
         buf.extend(format!("\ntype=EOE msg=audit({msgid}): \n").bytes());
+        buf
+    }
+
+    #[test]
+    fn strip_long_argv() -> Result<(), Box<dyn Error>> {
+        let ec: Rc<RefCell<Option<Event>>> = Rc::new(RefCell::new(None));
+
+        let mut c = Coalesce::new(mk_emit(&ec));
+
+        c.settings.execve_argv_limit_bytes = Some(10000);
+        let buf = gen_long_find_execve();
 
         process_record(&mut c, &buf)?;
         {
