@@ -9,7 +9,8 @@ use serde::{Serialize, Serializer};
 use thiserror::Error;
 
 use crate::label_matcher::LabelMatcher;
-use crate::types::EventID;
+
+use linux_audit_parser::*;
 
 #[cfg(all(feature = "procfs", target_os = "linux"))]
 use crate::procfs;
@@ -89,8 +90,10 @@ pub struct Process {
     /// parent's porocess ID
     pub ppid: u32,
     /// path to binary
+    #[serde(serialize_with = "serialize_name")]
     pub exe: Option<Vec<u8>>,
     /// process-settable argv[0]
+    #[serde(serialize_with = "serialize_name")]
     pub comm: Option<Vec<u8>>,
     /// Labels assigned to process
     pub labels: HashSet<Vec<u8>>,
@@ -98,6 +101,13 @@ pub struct Process {
     pub container_info: Option<ContainerInfo>,
 }
 
+fn serialize_name<S>(t: &Option<Vec<u8>>, s: S) -> Result<S::Ok, S::Error>
+where
+    S: Serializer,
+{
+    s.serialize_bytes(&t.clone().unwrap_or_default())
+}
+
 #[cfg(all(feature = "procfs", target_os = "linux"))]
 impl From<procfs::ProcPidInfo> for Process {
     fn from(p: procfs::ProcPidInfo) -> Self {
@@ -192,8 +202,9 @@ impl ProcTable {
     }
 
     pub fn insert(&mut self, proc: Process) {
-        self.processes.insert(proc.key, proc.clone());
-        self.current.insert(proc.pid, proc.key);
+        let (pid, key) = (proc.pid, proc.key);
+        self.processes.insert(proc.key, proc);
+        self.current.insert(pid, key);
     }
 
     /// Retrieves a process by key.
