@@ -1,13 +1,10 @@
 use std::ffi::{OsStr, OsString};
 use std::fs::{self, remove_file, rename, File, OpenOptions};
-use std::io::{Error, ErrorKind, Result, Seek, SeekFrom, Write};
+use std::io::{Error, Result, Seek, SeekFrom, Write};
 use std::os::unix::fs::OpenOptionsExt;
-use std::os::unix::io::AsRawFd;
 
 use exacl::{setfacl, AclEntry, Perm};
 
-use nix::sys::stat::{fchmod, Mode};
-
 /// A rotating (log) file writer
 ///
 /// [`FileRotate`] rotates the file after `filesize` bytes have been
@@ -90,9 +87,6 @@ impl FileRotate {
         }
 
         if let Ok(mut f) = OpenOptions::new().append(true).open(&self.basename) {
-            fchmod(f.as_raw_fd(), Mode::from_bits(0o600).unwrap())
-                .map_err(|e| Error::new(ErrorKind::Other, e))?;
-
             setfacl(&[&self.basename], &acl, None).map_err(|e| Error::new(e.kind(), e))?;
 
             self.offset = f.seek(SeekFrom::End(0))?;
@@ -131,7 +125,7 @@ impl Write for FileRotate {
         let mut f = self.file.as_ref().unwrap();
         let sz = f.write(buf)?;
         self.offset += sz as u64;
-        if self.offset > self.filesize && self.filesize != 0 {
+        if self.offset > self.filesize && self.filesize != 0 && buf.last() == Some(&b'\n') {
             f.sync_all()?;
             self.rotate()?;
         }
